
Array.prototype.remove = function(elem, all) {
  for (var i=this.length-1; i>=0; i--) {
    if (this[i] === elem) {
        this.splice(i, 1);
        if(!all)
          break;
    }
  }
  return this;
};

app.controller('configController', function($scope, $http) {
    $scope.configuration = {};
    
    $scope.originalConfig = {};
    
    $scope.dirtyFields = [];
    
    $scope.saveSuccessFlag = false;
    
    $scope.applySuccessFlag = false;
    
    $scope.error = false;
    
    $scope.errorMessage = '';
    
    $scope.xmldialog = false;
    
    $scope.showAllOptions = false;
    
    $scope.summary = {};
    
    $scope.minimize = true;
    
    $scope.init = function() {
    	$http.get('/wrench/config')
    	.success(function(data) {
    		console.log('Config retrieval successful, result: ' + data);
    	    $scope.error = false;
    	    $scope.errorMessage = '';
    	    $scope.configuration = data;
    	    $scope.originalConfig = {};
    	    for (var i in data) $scope.originalConfig[i] = data[i];
    	})
    	.error(function(data, status, headers, config) {
    		console.log('Config load error');
    		$scope.error = true;
    		$scope.errorMessage = 'Could not load config data from Wrench';
    		$scope.configuration = {};
    	  });
    	
    	$http.get('/wrench/dashboard/summary')
    	.success(function(data) {
    		console.log('Summary retrieval successful, result: ' + data);
    	    $scope.error = false;
    	    $scope.errorMessage = '';
    	    $scope.summary = data;
    	})
    	.error(function(data, status, headers, config) {
    		console.log('Summary error');
    		$scope.error = true;
    		$scope.errorMessage = 'Could not load summary data from Wrench';
    		$scope.summary = {};
    	  });
    };
    
    $scope.configChanged = function(configKey) {
    	
    	var displayConfigKey = configKey.split('_').join('.');
    	
    	console.log('Config key changed: ' + configKey);
    	
    	$scope.saveSuccessFlag = false;
    	
    	var originalValue = $scope.originalConfig[configKey];
    	console.log('Original value was: ' + originalValue);
    	console.log('Current value is: ' + $scope.configuration[configKey]);
    	
    	if(originalValue == $scope.configuration[configKey]) {
    		console.log('Removing dirty field: ' + displayConfigKey);
    		$scope.dirtyFields.remove(displayConfigKey);
    	} else {
    		if($scope.dirtyFields.indexOf(displayConfigKey) == -1) {
    			$scope.dirtyFields.push(displayConfigKey);
    		}
    	}
    }
    
    $scope.applyChanges = function() {
    	$scope.saveSuccessFlag = false;
    	console.log("Applying changes");
    	var deltaPack = $scope.getDeltaPack();
    	$http.post('/wrench/config', deltaPack).then(
    	  function(response) {
    		  console.log("Config change server response " + response.status);
    		  if(response.status == 200) {
    			  $scope.error = false;
    	    	  $scope.errorMessage = '';
    	    	  $scope.configuration = response.data;
    	    	  $scope.originalConfig = {};
    	    	  for (var i in response.data) $scope.originalConfig[i] = response.data[i];
    	    	  $scope.dirtyFields = [];
    	    	  $scope.applySuccessFlag = true;
    	    	  
    	    	  console.log("Saving changes after application was successful");
    	      	  $http.post('/wrench/save?minimize=' + $scope.minimize).then(
    	      			function(response) {
    	      				console.log("Config change server response " + response.status);
    	      				if(response.status == 200) {
    	      					$scope.error = false;
    	      					$scope.errorMessage = '';
    	      					$scope.applySuccessFlag = false;
    	      					$scope.saveSuccessFlag = false;
    	      				} else {
    	      					$scope.error = true;
    	      					$scope.errorMessage = response.data.errorMessage;
    	      					$scope.applySuccessFlag = false;
    	      					$scope.saveSuccessFlag = false;
    	      				}
    	      			},
    	      			function(response) {
    	      				$scope.error = true;
    	      				$scope.errorMessage = response.data.errorMessage;
    	      				$scope.applySuccessFlag = false;
    	      				$scope.saveSuccessFlag = false;
    	      			}
    	      	);
    	      	
    		  } else {
    			  $scope.error = true;
    			  $scope.errorMessage = response.data.errorMessage;
    			  $scope.applySuccessFlag = false;
    		  }
    	  },
    	  function(response) {
			  $scope.error = true;
			  $scope.errorMessage = response.data.errorMessage;
			  $scope.applySuccessFlag = false;
			  $scope.saveSuccessFlag = false;
    	  }
        );
    }
    
    $scope.saveChanges = function() {
    	$scope.applyChanges();
    }
    
    $scope.init();
    
    $scope.getDeltaPack = function() {
    	var pack = {};
    	for(var i = 0; i < $scope.dirtyFields.length; i++) {
    		pack[$scope.dirtyFields[i]] = $scope.configuration[$scope.dirtyFields[i].split('.').join('_')];
    	}
    	return pack;
    }
    
    $scope.exportFragment = function() {
    	$http.get('/wrench/export/xml')
    	.success(function(data) {
    		$scope.xmldialog = data;
    		console.log('Config expport successful, result: ' + data);
    	})
    	.error(function(data, status, headers, config) {
    		$scope.xmldialog = false;
    		console.log('Config export error');
    	});
    }
    
    $scope.v = function(property) {
    	return $scope.configuration[property.split('.').join('_')];
    }
    
});